@@ -1,8 +1,7 @@
 import pytest
 
-from xonsh.environ import Env
-from xonsh.parsers.completion_context import CompletionContextParser
 from xonsh.completers.environment import complete_environment_vars
+from xonsh.parsers.completion_context import CompletionContextParser
 
 
 @pytest.fixture(scope="session")
@@ -19,7 +18,7 @@ def parser():
     ),
 )
 def test_simple(cmd, xession, monkeypatch, parser):
-    monkeypatch.setattr(xession, "env", Env({"WOW": 1}))
+    xession.env.update({"WOW": 1})
 
     context = parser.parse(cmd, len(cmd))
     comps, lprefix = complete_environment_vars(context)
@@ -28,7 +27,7 @@ def test_simple(cmd, xession, monkeypatch, parser):
 
 
 def test_rich_completions(xession, monkeypatch, parser):
-    monkeypatch.setattr(xession, "env", Env({"WOW": 1}))
+    xession.env.update({"WOW": 1})
     xession.env.register("WOW", type=int, doc="Nice Docs!")
 
     context = parser.parse("$WO", 3)
