@@ -1,15 +1,17 @@
 import itertools
 import typing as tp
+from unittest import mock
 
 import pytest
 
+import xonsh.parsers.completion_context as ctx
 from xonsh.parsers.completion_context import (
     CommandArg,
     CommandContext,
     CompletionContextParser,
     PythonContext,
 )
-
+from xonsh.pytest.tools import ON_WINDOWS
 
 DEBUG = False
 MISSING = object()
@@ -21,8 +23,19 @@ PARSER: tp.Optional[CompletionContextParser] = None
 def parser():
     global PARSER
     PARSER = CompletionContextParser(debug=DEBUG)
+    patcher = None
+    if ON_WINDOWS:
+        # on-windows has an option for interactive sessions. Overriding the lazyObject
+        patcher = mock.patch.object(
+            ctx,
+            "LINE_CONT_REPLACEMENT_DIFF",
+            ("\\\n", "", -2),
+        )
+        patcher.start()
     yield
     PARSER = None
+    if ON_WINDOWS and patcher:
+        patcher.stop()
 
 
 def parse(command, inner_index):
