@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Testing xonsh import hooks"""
 import os
 from importlib import import_module
@@ -6,19 +5,13 @@ from importlib import import_module
 import pytest
 
 from xonsh import imphooks
-from xonsh.execer import Execer
-from xonsh.environ import Env
-from xonsh.built_ins import XSH
-
-imphooks.install_import_hooks()
 
 
 @pytest.fixture(autouse=True)
 def imp_env(xession):
-    Execer(unload=False)
-    xession.env = Env({"PATH": [], "PATHEXT": []})
+    xession.env.update({"PATH": [], "PATHEXT": []})
+    imphooks.install_import_hooks(xession.execer)
     yield
-    XSH.unload()
 
 
 def test_import():
@@ -67,5 +60,5 @@ def test_get_source():
     mod = import_module("sample")
     loader = mod.__loader__
     source = loader.get_source("sample")
-    with open(os.path.join(TEST_DIR, "sample.xsh"), "rt") as srcfile:
+    with open(os.path.join(TEST_DIR, "sample.xsh")) as srcfile:
         assert source == srcfile.read()
