@@ -1,18 +1,16 @@
-# -*- coding: utf-8 -*-
 """Tests the xonsh main function."""
-from __future__ import unicode_literals, print_function
-from contextlib import contextmanager
-
 import builtins
 import gc
 import os
 import os.path
 import sys
+from contextlib import contextmanager
+
+import pytest
 
 import xonsh.main
 from xonsh.main import XonshMode
-import pytest
-from tools import ON_WINDOWS, TEST_DIR, skip_if_on_windows
+from xonsh.pytest.tools import ON_WINDOWS, TEST_DIR, skip_if_on_windows
 
 
 def Shell(*args, **kwargs):
@@ -425,7 +423,7 @@ def test_xonsh_failback(
     rc_shells,
     exp_shell,
     shell,
-    xonsh_builtins,
+    xession,
     monkeypatch,
     monkeypatch_stderr,
 ):
@@ -489,8 +487,10 @@ def test_xonsh_failback_script_from_file(shell, monkeypatch, monkeypatch_stderr)
 
     script = os.path.join(TEST_DIR, "scripts", "raise.xsh")
     monkeypatch.setattr(sys, "argv", ["xonsh", script])
-    with pytest.raises(Exception):
-        xonsh.main.main()
+
+    # changed in #4662: User-Code exceptions are now caught in main and handled there
+    # => we expect that no exception is thrown
+
     assert len(checker) == 0
 
 
