@@ -1,18 +1,19 @@
 """This module provides the implementation for the retrieving completion results
 from bash.
 """
+import functools
+
 # developer note: this file should not perform any action on import.
 #                 This file comes from https://github.com/xonsh/py-bash-completion
 #                 and should be edited there!
 import os
-import re
-import sys
-import shlex
-import shutil
 import pathlib
 import platform
-import functools
+import re
+import shlex
+import shutil
 import subprocess
+import sys
 import typing as tp
 
 __version__ = "0.2.7"
@@ -110,7 +111,7 @@ def _get_bash_completions_source(paths=None):
         paths = _BASH_COMPLETIONS_PATHS_DEFAULT
     for path in map(pathlib.Path, paths):
         if path.is_file():
-            return 'source "{}"'.format(path.as_posix())
+            return f'source "{path.as_posix()}"'
     return None
 
 
@@ -185,7 +186,7 @@ def _bash_quote_paths(paths, start, end):
             start = end = _bash_quote_to_use(s)
         if os.path.isdir(_bash_expand_path(s)):
             _tail = slash
-        elif end == "":
+        elif end == "" and not s.endswith("="):
             _tail = space
         else:
             _tail = ""
@@ -292,7 +293,7 @@ def bash_completions(
     opening_quote="",
     closing_quote="",
     arg_index=None,
-    **kwargs
+    **kwargs,
 ):
     """Completes based on results from BASH completion.
 
@@ -315,7 +316,7 @@ def bash_completions(
         since it lazy-loads individual completion scripts. For both
         bash-completion v1.x and v2.x, paths of individual completion scripts
         (like ``.../completes/ssh``) do not need to be included here. The
-        default values are platform dependent, but sane.
+        default values are platform dependent, but reasonable.
     command : str or None, optional
         The /path/to/bash to use. If None, it will be selected based on the
         from the environment and platform.
@@ -408,6 +409,12 @@ def bash_completions(
 
     # Ensure input to `commonprefix` is a list (now required by Python 3.6)
     commprefix = os.path.commonprefix(list(out))
+
+    if prefix.startswith("~") and commprefix and prefix not in commprefix:
+        home_ = os.path.expanduser("~")
+        out = {f"~/{os.path.relpath(p, home_)}" for p in out}
+        commprefix = f"~/{os.path.relpath(commprefix, home_)}"
+
     strip_len = 0
     strip_prefix = prefix.strip("\"'")
     while strip_len < len(strip_prefix) and strip_len < len(commprefix):
@@ -418,7 +425,13 @@ def bash_completions(
     if "-o noquote" not in complete_stmt:
         out, need_quotes = quote_paths(out, opening_quote, closing_quote)
     if "-o nospace" in complete_stmt:
-        out = set([x.rstrip() for x in out])
+        out = {x.rstrip() for x in out}
+
+    # For arguments like 'status=progress', the completion script only returns
+    # the part after '=' in the completion results. This causes the strip_len
+    # to be incorrectly calculated, so it needs to be fixed here
+    if "=" in prefix and "=" not in commprefix:
+        strip_len = prefix.index("=") + 1
 
     return out, max(len(prefix) - strip_len, 0)
 
