@@ -2,18 +2,18 @@
 compatibility layers to make use of the 'best' implementation available
 on a platform.
 """
-import os
-import sys
+import collections.abc as cabc
 import ctypes  # noqa
-import signal
+import functools
+import importlib.util
+import os
 import pathlib
 import platform
-import functools
+import signal
 import subprocess
-import collections.abc as cabc
-import importlib.util
+import sys
 
-from xonsh.lazyasd import LazyBool, lazyobject, lazybool
+from xonsh.lazyasd import LazyBool, lazybool, lazyobject
 
 # do not import any xonsh-modules here to avoid circular dependencies
 
@@ -196,7 +196,7 @@ def ptk_above_min_supported():
 def win_ansi_support():
     if ON_WINDOWS:
         try:
-            from prompt_toolkit.utils import is_windows_vt100_supported, is_conemu_ansi
+            from prompt_toolkit.utils import is_conemu_ansi, is_windows_vt100_supported
         except ImportError:
             return False
         return is_conemu_ansi() or is_windows_vt100_supported()
@@ -303,7 +303,7 @@ CC = 6
 def githash():
     """Returns a tuple contains two strings: the hash and the date."""
     install_base = os.path.dirname(__file__)
-    githash_file = "{}/dev.githash".format(install_base)
+    githash_file = f"{install_base}/dev.githash"
     if not os.path.exists(githash_file):
         return None, None
     sha = None
@@ -417,7 +417,7 @@ if ON_WINDOWS:
         def __init__(self):
             import nt
 
-            self._upperkeys = dict((k.upper(), k) for k in nt.environ)
+            self._upperkeys = {k.upper(): k for k in nt.environ}
 
         def _sync(self):
             """Ensure that the case sensitive map of the keys are
@@ -494,8 +494,9 @@ def BASH_COMPLETIONS_DEFAULT():
     elif ON_DARWIN:
         bcd = (
             "/usr/local/share/bash-completion/bash_completion",  # v2.x
-            "/usr/local/etc/bash_completion",
-        )  # v1.x
+            "/usr/local/etc/bash_completion",  # v1.x
+            "/opt/homebrew/share/bash-completion/bash_completion",  # v2.x on M1
+        )
     elif ON_WINDOWS and git_for_windows_path():
         bcd = (
             os.path.join(
