@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """The readline based xonsh shell.
 
 Portions of this code related to initializing the readline library
@@ -10,49 +9,52 @@ are included from the IPython project.  The IPython project is:
 * Copyright (c) 2001, Nathaniel Gray <n8gray@caltech.edu>
 
 """
-import os
-import sys
 import cmd
+import collections
+import importlib
+import os
 import select
 import shutil
-import importlib
+import sys
 import threading
-import collections
+import typing as tp
 
 import xonsh.completers.tools as xct
-from xonsh.built_ins import XSH
-from xonsh.lazyasd import LazyObject, lazyobject
-from xonsh.base_shell import BaseShell
 from xonsh.ansi_colors import (
-    ansi_partial_color_format,
-    ansi_color_style_names,
     ansi_color_style,
+    ansi_color_style_names,
+    ansi_partial_color_format,
+)
+from xonsh.base_shell import BaseShell
+from xonsh.built_ins import XSH
+from xonsh.events import events
+from xonsh.lazyasd import LazyObject, lazyobject
+from xonsh.lazyimps import pyghooks, pygments, winutils
+from xonsh.platform import (
+    ON_CYGWIN,
+    ON_DARWIN,
+    ON_MSYS,
+    ON_POSIX,
+    ON_WINDOWS,
+    os_environ,
 )
 from xonsh.prompt.base import multiline_prompt
 from xonsh.tools import (
+    carriage_return,
+    columnize,
+    ends_with_colon_token,
     print_exception,
     to_bool,
-    columnize,
-    carriage_return,
-    ends_with_colon_token,
 )
-from xonsh.platform import (
-    ON_WINDOWS,
-    ON_CYGWIN,
-    ON_MSYS,
-    ON_DARWIN,
-    ON_POSIX,
-    os_environ,
-)
-from xonsh.lazyimps import pygments, pyghooks, winutils
-from xonsh.events import events
 
-readline = None
-RL_COMPLETION_SUPPRESS_APPEND = RL_LIB = RL_STATE = None
-RL_COMPLETION_QUERY_ITEMS = None
+if tp.TYPE_CHECKING:
+    from types import ModuleType
+
+readline: "ModuleType|None" = None
+RL_COMPLETION_SUPPRESS_APPEND = RL_LIB = RL_STATE = None  # type: tp.Any
+RL_COMPLETION_QUERY_ITEMS: "tp.Any" = None
 RL_CAN_RESIZE = False
-RL_DONE = None
-RL_VARIABLE_VALUE = None
+RL_VARIABLE_VALUE: "tp.Callable[..., tp.Any]|None" = None
 _RL_STATE_DONE = 0x1000000
 _RL_STATE_ISEARCH = 0x0000080
 
@@ -117,7 +119,7 @@ def setup_readline():
     readline.parse_and_bind('"\\e[B": history-search-forward')
     readline.parse_and_bind('"\\e[A": history-search-backward')
     # Setup Shift-Tab to indent
-    readline.parse_and_bind('"\\e[Z": "{0}"'.format(env.get("INDENT")))
+    readline.parse_and_bind('"\\e[Z": "{}"'.format(env.get("INDENT")))
 
     # handle tab completion differences found in libedit readline compatibility
     # as discussed at http://stackoverflow.com/a/7116997
@@ -405,7 +407,7 @@ class ReadlineShell(BaseShell, cmd.Cmd):
             return 1
         elif len(completions) <= XSH.env.get("COMPLETION_QUERY_LIMIT"):
             return 2
-        msg = "\nDisplay all {} possibilities? ".format(len(completions))
+        msg = f"\nDisplay all {len(completions)} possibilities? "
         msg += "({GREEN}y{RESET} or {RED}n{RESET})"
         self.print_color(msg, end="", flush=True, file=sys.stderr)
         yn = "x"
@@ -701,7 +703,7 @@ class ReadlineHistoryAdder(threading.Thread):
         current readline instance. May wait for the history garbage
         collector to finish.
         """
-        super(ReadlineHistoryAdder, self).__init__(*args, **kwargs)
+        super().__init__(*args, **kwargs)
         self.daemon = True
         self.wait_for_gc = wait_for_gc
         self.start()
