@@ -14,14 +14,15 @@ Original file credits:
                   'Michael Foord')
 """
 
-import re
-import io
-import sys
-import codecs
 import builtins
-import itertools
+import codecs
 import collections
+import io
+import itertools
+import re
+import sys
 import token
+import typing as tp
 from token import (
     AMPER,
     AMPEREQUAL,
@@ -52,11 +53,11 @@ from token import (
     LSQB,
     MINEQUAL,
     MINUS,
+    N_TOKENS,
     NAME,
     NEWLINE,
     NOTEQUAL,
     NUMBER,
-    N_TOKENS,
     OP,
     PERCENT,
     PERCENTEQUAL,
@@ -78,7 +79,6 @@ from token import (
     VBAREQUAL,
     tok_name,
 )
-import typing as tp
 
 from xonsh.lazyasd import LazyObject
 from xonsh.platform import PYTHON_VERSION_INFO
@@ -93,7 +93,7 @@ cookie_re = LazyObject(
     "cookie_re",
 )
 blank_re = LazyObject(
-    lambda: re.compile(br"^[ \t\f]*(?:[#\r\n]|$)", re.ASCII), globals(), "blank_re"
+    lambda: re.compile(rb"^[ \t\f]*(?:[#\r\n]|$)", re.ASCII), globals(), "blank_re"
 )
 
 #
@@ -114,6 +114,8 @@ __all__ = token.__all__ + [  # type:ignore
     "ATEQUAL",
     "DOLLARNAME",
     "IOREDIRECT",
+    "MATCH",
+    "CASE",
 ]
 HAS_ASYNC = PYTHON_VERSION_INFO < (3, 7, 0)
 if HAS_ASYNC:
@@ -151,6 +153,12 @@ N_TOKENS += 1
 ATEQUAL = N_TOKENS
 tok_name[N_TOKENS] = "ATEQUAL"
 N_TOKENS += 1
+MATCH = N_TOKENS
+tok_name[N_TOKENS] = "MATCH"
+N_TOKENS += 1
+CASE = N_TOKENS
+tok_name[N_TOKENS] = "CASE"
+N_TOKENS += 1
 _xonsh_tokens = {
     "?": "QUESTION",
     "@=": "ATEQUAL",
@@ -165,6 +173,8 @@ _xonsh_tokens = {
     "${": "DOLLARLBRACE",
     "??": "DOUBLEQUESTION",
     "@$(": "ATDOLLARLPAREN",
+    "match": "MATCH",
+    "case": "CASE",
 }
 
 additional_parenlevs = frozenset({"@(", "!(", "![", "$(", "$[", "${", "@$("})
@@ -329,11 +339,11 @@ _redir_map = (
     "o>2",
     "1>2",
 )
-IORedirect = group(group(*_redir_map), "{}>>?".format(group(*_redir_names)))
+IORedirect = group(group(*_redir_map), f"{group(*_redir_names)}>>?")
 
 _redir_check_0 = set(_redir_map)
-_redir_check_1 = {"{}>".format(i) for i in _redir_names}.union(_redir_check_0)
-_redir_check_2 = {"{}>>".format(i) for i in _redir_names}.union(_redir_check_1)
+_redir_check_1 = {f"{i}>" for i in _redir_names}.union(_redir_check_0)
+_redir_check_2 = {f"{i}>>" for i in _redir_names}.union(_redir_check_1)
 _redir_check = frozenset(_redir_check_2)
 
 Operator = group(
@@ -788,7 +798,7 @@ def detect_encoding(readline):
         except UnicodeDecodeError:
             msg = "invalid or missing encoding declaration"
             if filename is not None:
-                msg = "{} for {!r}".format(msg, filename)
+                msg = f"{msg} for {filename!r}"
             raise SyntaxError(msg)
 
         match = cookie_re.match(line_string)
@@ -802,7 +812,7 @@ def detect_encoding(readline):
             if filename is None:
                 msg = "unknown encoding: " + encoding
             else:
-                msg = "unknown encoding for {!r}: {}".format(filename, encoding)
+                msg = f"unknown encoding for {filename!r}: {encoding}"
             raise SyntaxError(msg)
 
         if bom_found:
@@ -811,7 +821,7 @@ def detect_encoding(readline):
                 if filename is None:
                     msg = "encoding problem: utf-8"
                 else:
-                    msg = "encoding problem for {!r}: utf-8".format(filename)
+                    msg = f"encoding problem for {filename!r}: utf-8"
                 raise SyntaxError(msg)
             encoding += "-sig"
         return encoding
@@ -1177,7 +1187,7 @@ def tokenize_main():
             args = (filename,) + location + (message,)
             perror("%s:%d:%d: error: %s" % args)
         elif filename:
-            perror("%s: error: %s" % (filename, message))
+            perror(f"{filename}: error: {message}")
         else:
             perror("error: %s" % message)
         sys.exit(1)
