@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """Misc. xonsh tools.
 
 The following implementations were forked from the IPython project:
@@ -17,6 +16,7 @@ Implementations:
 * indent()
 
 """
+import ast
 import collections
 import collections.abc as cabc
 import contextlib
@@ -25,26 +25,26 @@ import datetime
 import functools
 import glob
 import itertools
+import operator
 import os
 import pathlib
 import re
+import shlex
+import string
 import subprocess
 import sys
 import threading
 import traceback
-import warnings
-import operator
-import ast
-import string
 import typing as tp
-import shlex
+import warnings
 
 # adding imports from further xonsh modules is discouraged to avoid circular
 # dependencies
 from xonsh import __version__
-from xonsh.lazyasd import LazyObject, LazyDict, lazyobject
+from xonsh.lazyasd import LazyDict, LazyObject, lazyobject
 from xonsh.platform import (
     DEFAULT_ENCODING,
+    HAS_PYGMENTS,
     ON_LINUX,
     ON_WINDOWS,
     expanduser,
@@ -83,7 +83,7 @@ class XonshCalledProcessError(XonshError, subprocess.CalledProcessError):
     returncode of the command is nonzero.
 
     Example:
-    -------
+    --------
         try:
             for line in !(ls):
                 print(line)
@@ -289,12 +289,28 @@ class EnvPath(cabc.MutableSequence):
             self._l.insert(0 if front else len(self._l), data)
 
 
+class FlexibleFormatter(string.Formatter):
+    """Support nested fields inside conditional formatters
+
+    e.g. template ``{user:| {RED}{}{RESET}}`` will become ``| {RED}user{RESET}`` when user=user.
+    """
+
+    def get_value(self, key: "int|str", args, kwargs) -> str:
+        if isinstance(key, int):
+            return args[key]
+        else:
+            if key in kwargs:
+                return kwargs[key]
+            # in case of colors, this will work without nested braces
+            return "{" + key + "}"
+
+
 @lazyobject
 def FORMATTER():
-    return string.Formatter()
+    return FlexibleFormatter()
 
 
-class DefaultNotGivenType(object):
+class DefaultNotGivenType:
     """Singleton for representing when no default value is given."""
 
     __inst: tp.Optional["DefaultNotGivenType"] = None
@@ -542,10 +558,12 @@ def get_line_continuation():
     mode on Windows the backslash must be preceded by a space. This is because
     paths on Windows may end in a backslash.
     """
-    if ON_WINDOWS and hasattr(xsh, "env") and xsh.env.get("XONSH_INTERACTIVE", False):
-        return " \\"
-    else:
-        return "\\"
+    if ON_WINDOWS:
+        env = getattr(xsh, "env", None) or {}
+        if env.get("XONSH_INTERACTIVE", False):
+            return " \\"
+
+    return "\\"
 
 
 def get_logical_line(lines, idx):
@@ -904,7 +922,7 @@ def suggest_commands(cmd, env):
     for _cmd in xsh.commands_cache.all_commands:
         if _cmd not in suggested:
             if levenshtein(_cmd.lower(), cmd, thresh) < thresh:
-                suggested[_cmd] = "Command ({0})".format(_cmd)
+                suggested[_cmd] = f"Command ({_cmd})"
 
     suggested = collections.OrderedDict(
         sorted(
@@ -917,13 +935,13 @@ def suggest_commands(cmd, env):
         rtn = command_not_found(cmd, env)
     else:
         oneof = "" if num == 1 else "one of "
-        tips = "Did you mean {}the following?".format(oneof)
+        tips = f"Did you mean {oneof}the following?"
         items = list(suggested.popitem(False) for _ in range(num))
         length = max(len(key) for key, _ in items) + 2
         alternatives = "\n".join(
             "    {: <{}} {}".format(key + ":", length, val) for key, val in items
         )
-        rtn = "{}\n{}".format(tips, alternatives)
+        rtn = f"{tips}\n{alternatives}"
         c = command_not_found(cmd, env)
         rtn += ("\n\n" + c) if len(c) > 0 else ""
     return rtn
@@ -978,8 +996,39 @@ def print_warning(msg):
     sys.stderr.write(msg)
 
 
-def print_exception(msg=None):
-    """Print exceptions with/without traceback."""
+def print_exception(msg=None, exc_info=None):
+    """Print given exception (or current if None) with/without traceback and set sys.last_type, sys.last_value, sys.last_traceback accordingly."""
+
+    # is no exec_info() triple is given, use the exception beeing handled at the moment
+    if exc_info is None:
+        exc_info = sys.exc_info()
+
+    # these values (initialized with their default for traceback.print_exception) control how an exception is printed
+    limit = None
+    chain = True
+
+    _, debug_level = _get_manual_env_var("XONSH_DEBUG", 0)
+
+    # the interal state of the parsers stack is
+    # not helpful in normal operation (XONSH_DEBUG == 0).
+    # this is also done to be consistent with python
+    is_syntax_error = issubclass(exc_info[0], SyntaxError)
+
+    # XonshErrors don't show where in the users code they occured
+    # (most are reported deeper in the callstack, e.g. see procs/pipelines.py),
+    # but only show non-helpful xonsh internals.
+    # These are only relevent when developing/debugging xonsh itself.
+    # Therefore, dont print these traces until this gets overhauled.
+    is_xonsh_error = exc_info[0] in (XonshError, XonshCalledProcessError)
+
+    # hide unhelpful traces if not debugging
+    hide_stacktrace = debug_level == 0 and (is_syntax_error or is_xonsh_error)
+    if hide_stacktrace:
+        limit = 0
+        chain = False
+
+    sys.last_type, sys.last_value, sys.last_traceback = exc_info
+
     manually_set_trace, show_trace = _get_manual_env_var("XONSH_SHOW_TRACEBACK", False)
     manually_set_logfile, log_file = _get_manual_env_var("XONSH_TRACEBACK_LOGFILE")
     if (not manually_set_trace) and (not manually_set_logfile):
@@ -1000,7 +1049,24 @@ def print_exception(msg=None):
                 "xonsh: To log full traceback to a file set: "
                 "$XONSH_TRACEBACK_LOGFILE = <filename>\n"
             )
-        traceback.print_exc()
+
+        traceback_str = "".join(
+            traceback.format_exception(*exc_info, limit=limit, chain=chain)
+        )
+
+        # color the traceback if available
+        _, interactive = _get_manual_env_var("XONSH_INTERACTIVE", 0)
+        _, color_results = _get_manual_env_var("COLOR_RESULTS", 0)
+        if interactive and color_results and HAS_PYGMENTS:
+            import pygments.lexers.python
+
+            lexer = pygments.lexers.python.PythonTracebackLexer()
+            tokens = list(pygments.lex(traceback_str, lexer=lexer))
+            # this goes to stdout, but since we are interactive it doesn't matter
+            print_color(tokens, end="")
+        else:
+            print(traceback_str, file=sys.stderr, end="")
+
     # additionally, check if a file for traceback logging has been
     # specified and convert to a proper option if needed
     log_file = to_logfile_opt(log_file)
@@ -1008,22 +1074,22 @@ def print_exception(msg=None):
         # if log_file <> '' or log_file <> None, append
         # traceback log there as well
         with open(os.path.abspath(log_file), "a") as f:
-            traceback.print_exc(file=f)
+            traceback.print_exception(*exc_info, limit=limit, chain=chain, file=f)
 
     if not show_trace:
         # if traceback output is disabled, print the exception's
         # error message on stderr.
-        display_error_message()
+        display_error_message(exc_info)
     if msg:
         msg = msg if msg.endswith("\n") else msg + "\n"
         sys.stderr.write(msg)
 
 
-def display_error_message(strip_xonsh_error_types=True):
+def display_error_message(exc_info, strip_xonsh_error_types=True):
     """
-    Prints the error message of the current exception on stderr.
+    Prints the error message of the given sys.exc_info() triple on stderr.
     """
-    exc_type, exc_value, exc_traceback = sys.exc_info()
+    exc_type, exc_value, exc_traceback = exc_info
     exception_only = traceback.format_exception_only(exc_type, exc_value)
     if exc_type is XonshError and strip_xonsh_error_types:
         exception_only[0] = exception_only[0].partition(": ")[-1]
@@ -1225,8 +1291,8 @@ def always_none(x):
 
 
 def ensure_string(x):
-    """Returns a string if x is not a string, and x if it already is."""
-    return str(x)
+    """Returns a string if x is not a string, and x if it already is. If x is None, the empty string is returned."""
+    return str(x) if x is not None else ""
 
 
 def is_path(x):
@@ -1241,11 +1307,10 @@ def is_env_path(x):
 
 def str_to_path(x):
     """Converts a string to a path."""
-    if x is None:
+    if x is None or x == "":
         return None
     elif isinstance(x, str):
-        # checking x is needed to avoid uncontrolled converting empty string to Path('.')
-        return pathlib.Path(x) if x else None
+        return pathlib.Path(x)
     elif isinstance(x, pathlib.Path):
         return x
     elif isinstance(x, EnvPath) and len(x) == 1:
@@ -1266,7 +1331,7 @@ def str_to_env_path(x):
 
 def path_to_str(x):
     """Converts a path to a string."""
-    return str(x)
+    return str(x) if x is not None else ""
 
 
 def env_path_to_str(x):
@@ -1307,7 +1372,7 @@ def to_logfile_opt(x):
     if isinstance(x, os.PathLike):  # type: ignore
         x = str(x)
     if is_logfile_opt(x):
-        return x
+        return expand_path(x) if x else x
     else:
         # if option is not valid, return a proper
         # option and inform the user on stderr
@@ -1429,6 +1494,33 @@ def SLICE_REG():
     )
 
 
+def to_shlvl(x):
+    """Converts a value to an $SHLVL integer according to bash's behaviour (variables.c::adjust_shell_level)."""
+    if x is None:
+        return 0
+    else:
+        x = str(x)
+    try:
+        return adjust_shlvl(max(0, int(x)), 0)
+    except ValueError:
+        return 0
+
+
+def is_valid_shlvl(x):
+    """Checks whether a variable is a proper $SHLVL integer."""
+    return isinstance(x, int) and to_shlvl(x) == x
+
+
+def adjust_shlvl(old_lvl: int, change: int):
+    """Adjusts an $SHLVL integer according to bash's behaviour (variables.c::adjust_shell_level)."""
+    new_level = old_lvl + change
+    if new_level < 0:
+        new_level = 0
+    elif new_level >= 1000:
+        new_level = 1
+    return new_level
+
+
 def ensure_slice(x):
     """Try to convert an object into a slice, complain on failure"""
     if not x and x != 0:
@@ -1448,12 +1540,12 @@ def ensure_slice(x):
             groups = (int(i) if i else None for i in m.groups())
             s = slice(*groups)
         else:
-            raise ValueError("cannot convert {!r} to slice".format(x))
+            raise ValueError(f"cannot convert {x!r} to slice")
     except TypeError:
         try:
             s = slice(*(int(i) for i in x))
         except (TypeError, ValueError):
-            raise ValueError("cannot convert {!r} to slice".format(x))
+            raise ValueError(f"cannot convert {x!r} to slice")
     return s
 
 
@@ -1616,7 +1708,7 @@ def ptk2_color_depth_setter(x):
     elif x in {"", None}:
         x = ""
     else:
-        msg = '"{}" is not a valid value for $PROMPT_TOOLKIT_COLOR_DEPTH. '.format(x)
+        msg = f'"{x}" is not a valid value for $PROMPT_TOOLKIT_COLOR_DEPTH. '
         warnings.warn(msg, RuntimeWarning)
         x = ""
     if x == "" and "PROMPT_TOOLKIT_COLOR_DEPTH" in os_environ:
@@ -1641,7 +1733,7 @@ def to_completions_display_value(x):
     elif x in {"single", "readline"}:
         pass
     else:
-        msg = '"{}" is not a valid value for $COMPLETIONS_DISPLAY. '.format(x)
+        msg = f'"{x}" is not a valid value for $COMPLETIONS_DISPLAY. '
         msg += 'Using "multi".'
         warnings.warn(msg, RuntimeWarning)
         x = "multi"
@@ -1689,7 +1781,7 @@ def to_dict(x):
     try:
         x = ast.literal_eval(x)
     except (ValueError, SyntaxError):
-        msg = '"{}" can not be converted to Python dictionary.'.format(x)
+        msg = f'"{x}" can not be converted to Python dictionary.'
         warnings.warn(msg, RuntimeWarning)
         x = dict()
     return x
@@ -1701,7 +1793,7 @@ def to_str_str_dict(x):
         return x
     x = to_dict(x)
     if not is_str_str_dict(x):
-        msg = '"{}" can not be converted to str:str dictionary.'.format(x)
+        msg = f'"{x}" can not be converted to str:str dictionary.'
         warnings.warn(msg, RuntimeWarning)
         x = dict()
     return x
@@ -1862,7 +1954,7 @@ def to_history_tuple(x):
 
 def history_tuple_to_str(x):
     """Converts a valid history tuple to a canonical string."""
-    return "{0} {1}".format(*x)
+    return "{} {}".format(*x)
 
 
 def all_permutations(iterable):
@@ -2040,9 +2132,7 @@ def _win10_color_map():
         "ansibrightyellow": (249, 241, 165),
         "ansiwhite": (242, 242, 242),
     }
-    return {
-        k: "#{0:02x}{1:02x}{2:02x}".format(r, g, b) for k, (r, g, b) in cmap.items()
-    }
+    return {k: f"#{r:02x}{g:02x}{b:02x}" for k, (r, g, b) in cmap.items()}
 
 
 WIN10_COLOR_MAP = LazyObject(_win10_color_map, globals(), "WIN10_COLOR_MAP")
@@ -2131,11 +2221,7 @@ def intensify_colors_on_win_setter(enable):
     environment variable.
     """
     enable = to_bool(enable)
-    if (
-        hasattr(xsh, "shell")
-        and xsh.shell is not None
-        and hasattr(xsh.shell.shell.styler, "style_name")
-    ):
+    if xsh.shell is not None and hasattr(xsh.shell.shell.styler, "style_name"):
         delattr(xsh.shell.shell.styler, "style_name")
     return enable
 
@@ -2150,8 +2236,8 @@ def format_std_prepost(template, env=None):
     invis = "\001\002"
     if xsh.shell is None:
         # shell hasn't fully started up (probably still in xonshrc)
-        from xonsh.prompt.base import PromptFormatter
         from xonsh.ansi_colors import ansi_partial_color_format
+        from xonsh.prompt.base import PromptFormatter
 
         pf = PromptFormatter()
         s = pf(template)
@@ -2373,7 +2459,7 @@ def backup_file(fname):
 
     base, ext = os.path.splitext(fname)
     timestamp = datetime.now().strftime("%Y-%m-%d-%H-%M-%S-%f")
-    newfname = "%s.%s%s" % (base, timestamp, ext)
+    newfname = f"{base}.{timestamp}{ext}"
     shutil.move(fname, newfname)
 
 
@@ -2416,11 +2502,11 @@ def expand_case_matching(s):
         elif c.isalpha():
             folded = c.casefold()
             if len(folded) == 1:
-                c = "[{0}{1}]".format(c.upper(), c.lower())
+                c = f"[{c.upper()}{c.lower()}]"
             else:
-                newc = ["[{0}{1}]?".format(f.upper(), f.lower()) for f in folded[:-1]]
+                newc = [f"[{f.upper()}{f.lower()}]?" for f in folded[:-1]]
                 newc = "".join(newc)
-                newc += "[{0}{1}{2}]".format(folded[-1].upper(), folded[-1].lower(), c)
+                newc += f"[{folded[-1].upper()}{folded[-1].lower()}{c}]"
                 c = newc
         t.append(c)
     return "".join(t)
@@ -2614,7 +2700,7 @@ def deprecated(deprecated_in=None, removed_in=None):
         message_suffix = ""
 
     def decorated(func):
-        warning_message = "{} has been deprecated".format(func.__name__)
+        warning_message = f"{func.__name__} has been deprecated"
         warning_message += message_suffix
 
         @functools.wraps(func)
@@ -2624,7 +2710,7 @@ def deprecated(deprecated_in=None, removed_in=None):
             warnings.warn(warning_message, DeprecationWarning)
 
         wrapped.__doc__ = (
-            "{}\n\n{}".format(wrapped.__doc__, warning_message)
+            f"{wrapped.__doc__}\n\n{warning_message}"
             if wrapped.__doc__
             else warning_message
         )
@@ -2640,9 +2726,9 @@ def _deprecated_message_suffix(deprecated_in, removed_in):
             deprecated_in, removed_in
         )
     elif deprecated_in and not removed_in:
-        message_suffix = " in version {}".format(deprecated_in)
+        message_suffix = f" in version {deprecated_in}"
     elif not deprecated_in and removed_in:
-        message_suffix = " and will be removed in version {}".format(removed_in)
+        message_suffix = f" and will be removed in version {removed_in}"
     else:
         message_suffix = None
 
@@ -2650,48 +2736,19 @@ def _deprecated_message_suffix(deprecated_in, removed_in):
 
 
 def _deprecated_error_on_expiration(name, removed_in):
-    from distutils.version import LooseVersion
+    from packaging.version import Version
 
     if not removed_in:
         return
-    elif LooseVersion(__version__) >= LooseVersion(removed_in):
-        raise AssertionError(
-            "{} has passed its version {} expiry date!".format(name, removed_in)
-        )
+    elif Version(__version__) >= Version(removed_in):
+        raise AssertionError(f"{name} has passed its version {removed_in} expiry date!")
 
 
 def to_repr_pretty_(inst, p, cycle):
-    name = "{0}.{1}".format(inst.__class__.__module__, inst.__class__.__name__)
+    name = f"{inst.__class__.__module__}.{inst.__class__.__name__}"
     with p.group(0, name + "(", ")"):
         if cycle:
             p.text("...")
         elif len(inst):
             p.break_()
             p.pretty(dict(inst))
-
-
-class XAttr:
-    """hold attribute and value"""
-
-    __slots__ = ("name", "value")
-
-    def __init__(self, val) -> None:
-        self.value = val
-
-    def __set_name__(self, owner, name) -> None:
-        self.name: str = name
-
-    def __get__(self, instance, owner) -> "XAttr":
-        return self
-
-    def __str__(self) -> str:
-        return f"<{self.name}={self.value}>"
-
-
-class NamedConstantMeta(type):
-    """utility class to hold list of values as class-attributes"""
-
-    def __iter__(cls) -> tp.Iterator[XAttr]:
-        for attr in vars(cls):
-            if not attr.startswith("__"):
-                yield getattr(cls, attr)
