@@ -1,56 +1,54 @@
 """The xonsh configuration (xonfig) utility."""
-import os
-import re
 import ast
-import sys
-import json
-import shutil
-import random
-import pprint
-import tempfile
-import textwrap
-import itertools
-import contextlib
 import collections
+import contextlib
+import itertools
+import json
+import os
+import pprint
+import random
+import re
+import shutil
+import textwrap
 import typing as tp
 
-from xonsh.ply import ply
-
 import xonsh.wizard as wiz
 from xonsh import __version__ as XONSH_VERSION
 from xonsh.built_ins import XSH
-from xonsh.cli_utils import ArgParserAlias, Annotated, Arg, add_args
-from xonsh.prompt.base import is_template_string
+from xonsh.cli_utils import Annotated, Arg, ArgParserAlias, add_args
+from xonsh.events import events
+from xonsh.foreign_shells import CANON_SHELL_NAMES
+from xonsh.lazyasd import lazyobject
 from xonsh.platform import (
-    is_readline_available,
-    ptk_version,
-    PYTHON_VERSION_INFO,
-    pygments_version,
-    ON_POSIX,
+    DEFAULT_ENCODING,
+    ON_CYGWIN,
+    ON_DARWIN,
     ON_LINUX,
-    linux_distro,
+    ON_MSYS,
+    ON_POSIX,
+    ON_WINDOWS,
     ON_WSL,
     ON_WSL1,
-    ON_DARWIN,
-    ON_WINDOWS,
-    ON_CYGWIN,
-    DEFAULT_ENCODING,
-    ON_MSYS,
+    PYTHON_VERSION_INFO,
     githash,
+    is_readline_available,
+    linux_distro,
+    ptk_version,
+    pygments_version,
 )
+from xonsh.ply import ply
+from xonsh.prompt.base import is_template_string
 from xonsh.tools import (
-    to_bool,
-    is_string,
-    print_exception,
-    is_superuser,
-    color_style_names,
-    print_color,
     color_style,
+    color_style_names,
+    is_string,
+    is_superuser,
+    print_color,
+    print_exception,
+    to_bool,
 )
-from xonsh.foreign_shells import CANON_SHELL_NAMES
 from xonsh.xontribs import find_xontrib, xontribs_loaded
-from xonsh.xontribs_meta import get_xontribs, Xontrib
-from xonsh.lazyasd import lazyobject
+from xonsh.xontribs_meta import Xontrib, get_xontribs
 
 HR = "'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'`-.,_,.-*'"
 WIZARD_HEAD = """
@@ -145,7 +143,15 @@ _XONFIG_SOURCE_FOREIGN_SHELL_COMMAND: tp.Dict[str, str] = collections.defaultdic
     lambda: "source-foreign", bash="source-bash", cmd="source-cmd", zsh="source-zsh"
 )
 
-XONSH_JUPYTER_KERNEL = "xonsh"
+
+events.doc(
+    "on_xonfig_info_requested",
+    """
+on_xonfig_info_requested() -> list[tuple[str, str]]
+
+Register a callable that will return extra info when ``xonfig info`` is called.
+""",
+)
 
 
 def _dump_xonfig_foreign_shell(path, value):
@@ -193,11 +199,11 @@ def _dump_xonfig_env(path, value):
     detyper = XSH.env.get_detyper(name)
     dval = str(value) if detyper is None else detyper(value)
     dval = str(value) if dval is None else dval
-    return "${name} = {val!r}".format(name=name, val=dval)
+    return f"${name} = {dval!r}"
 
 
 def _dump_xonfig_xontribs(path, value):
-    return "xontrib load {0}".format(" ".join(value))
+    return "xontrib load {}".format(" ".join(value))
 
 
 @lazyobject
@@ -559,15 +565,9 @@ def _info(
             ("encoding errors", env.get("XONSH_ENCODING_ERRORS")),
         ]
     )
-    jup_ksm = jup_kernel = None
-    try:
-        from jupyter_client.kernelspec import KernelSpecManager
-
-        jup_ksm = KernelSpecManager()
-        jup_kernel = jup_ksm.find_kernel_specs().get(XONSH_JUPYTER_KERNEL)
-    except Exception:
-        pass
-    data.extend([("on jupyter", jup_ksm is not None), ("jupyter kernel", jup_kernel)])
+    for p in XSH.builtins.events.on_xonfig_info_requested.fire():
+        if p is not None:
+            data.extend(p)
 
     data.extend([("xontrib", xontribs_loaded())])
     data.extend([("RC file", XSH.rc_files)])
@@ -666,7 +666,7 @@ def _colors(
 
     if style is not None:
         if style not in color_style_names():
-            print("Invalid style: {}".format(style))
+            print(f"Invalid style: {style}")
             return
         XSH.env["XONSH_COLOR_STYLE"] = style
 
@@ -698,95 +698,25 @@ def _web(
 
     Parameters
     ----------
-    browser : --nb, --no-browser
+    browser : --nb, --no-browser, -n
         don't open browser
     """
 
-    import subprocess
+    from xonsh.webconfig import main
 
-    subprocess.run([sys.executable, "-m", "xonsh.webconfig"] + _args[1:])
-
-
-def _jupyter_kernel(
-    user=False,
-    prefix=None,
-    root=None,
-):
-    """Generate xonsh kernel for jupyter.
-
-    Parameters
-    ----------
-    user : -u, --user
-        Install kernel spec in user config directory.
-    prefix : -p, --prefix
-        Installation prefix for bin, lib, etc.
-    root : -r, --root
-        Install relative to this alternate root directory.
-    """
-    try:
-        from jupyter_client.kernelspec import KernelSpecManager, NoSuchKernel
-    except ImportError as e:
-        raise ImportError("Jupyter not found in current Python environment") from e
-
-    ksm = KernelSpecManager()
-
-    prefix = prefix or sys.prefix
-    spec = {
-        "argv": [
-            sys.executable,
-            "-m",
-            "xonsh.jupyter_kernel",
-            "-f",
-            "{connection_file}",
-        ],
-        "display_name": "Xonsh",
-        "language": "xonsh",
-        "codemirror_mode": "shell",
-    }
-
-    if root and prefix:
-        # os.path.join isn't used since prefix is probably absolute
-        prefix = root + prefix
-
-    try:
-        old_jup_kernel = ksm.get_kernel_spec(XONSH_JUPYTER_KERNEL)
-        if not old_jup_kernel.resource_dir.startswith(prefix):
-            print(
-                "Removing existing Jupyter kernel found at {0}".format(
-                    old_jup_kernel.resource_dir
-                )
-            )
-        ksm.remove_kernel_spec(XONSH_JUPYTER_KERNEL)
-    except NoSuchKernel:
-        pass
-
-    if sys.platform == "win32":
-        # Ensure that conda-build detects the hard coded prefix
-        spec["argv"][0] = spec["argv"][0].replace(os.sep, os.altsep)
-        prefix = prefix.replace(os.sep, os.altsep)
-
-    with tempfile.TemporaryDirectory() as d:
-        os.chmod(d, 0o755)  # Starts off as 700, not user readable
-        with open(os.path.join(d, "kernel.json"), "w") as f:
-            json.dump(spec, f, sort_keys=True)
-
-        print("Installing Jupyter kernel spec:")
-        print("  root: {0!r}".format(root))
-        if user:
-            print("  as user: {0}".format(user))
-        elif root and prefix:
-            print("  combined prefix {0!r}".format(prefix))
-        else:
-            print("  prefix: {0!r}".format(prefix))
-        ksm.install_kernel_spec(
-            d, XONSH_JUPYTER_KERNEL, user=user, prefix=(None if user else prefix)
-        )
-        return 0
+    main.main(_args[1:])
 
 
 class XonfigAlias(ArgParserAlias):
     """Manage xonsh configuration."""
 
+    def __init__(self, **kwargs):
+        super().__init__(**kwargs)
+        self.extra_commands = []
+
+    def add_command(self, fn):
+        self.extra_commands.append(fn)
+
     def build(self):
         parser = self.create_parser(prog="xonfig")
         # register as default action
@@ -797,12 +727,13 @@ class XonfigAlias(ArgParserAlias):
         parser.add_command(_styles)
         parser.add_command(_colors)
         parser.add_command(_tutorial)
-        parser.add_command(_jupyter_kernel)
+        for fn in self.extra_commands:
+            parser.add_command(fn)
 
         return parser
 
 
-xonfig_main = XonfigAlias()
+xonfig_main = XonfigAlias(threadable=False)
 
 
 @lazyobject
